@@ -33,9 +33,14 @@ class search(ranaModule):
     self.localSearchResults = None # GLS results from onlineServices
     self.scroll = 0
     self.list = None # processed results: (distancefrom pos, rusult, absolut index)
+    self.maxIndex = 0 # based on the number of items in the list
     self.where='position'
     self.menuWatchId = None
     self.filters = {}
+
+  def firstTime(self):
+    self.menuWatchId = self.modrana.watch('menu', self._checkMenuEnteredCB)
+
   def loadFilters(self):
     """fill the filter directory
     the different categories are all represented by ordered dictionaries"""
@@ -147,7 +152,7 @@ class search(ranaModule):
       if(self.scroll > 0):
         self.scroll -= 1
         self.set("needRedraw", True)
-    elif(message == "down"):
+    elif(message == "down") and self.scroll < self.maxIndex:
       print "down"
       self.scroll += 1
       self.set("needRedraw", True)
@@ -220,11 +225,17 @@ class search(ranaModule):
 #        self.localSearchResults = local
 
     elif message == "customQuery":
-      # start input for custom query
+      # start text input for custom query
       entry = self.m.get('textEntry', None)
       if entry:
         entryText = ""
-        entry.entryBox(self, 'customQuery','Enter your search query',entryText)
+        entry.entryBox(self, 'customQuery','Enter your search query',entryText, persistentKey="lastCustomLocalSearchQuery")
+
+    elif message == "searchAddress":
+      # start text input for an address
+      entry = self.m.get('textEntry', None)
+      if entry:
+        entry.entryBox(self, 'address',description='Enter and address or location description', persistentKey="lastAddressSearchInput")
 
     elif message == "routeToActiveResult":
       """get a route from current position to active search result
@@ -257,23 +268,33 @@ class search(ranaModule):
       # * scroll down
       menus.drawButton(cr, x3, y3, dx, dy, "", "down_list", "%s:down" % self.moduleName)
       
-      list = self.updateDistance()
+      resultList = self.updateDistance()
+      # update maxIndex, needed for proper listing
+      if resultList:
+        self.maxIndex=len(resultList)-1
+
 
       if self.get('GLSOrdering', 'default') == 'distance': # if ordering by distance is turned on, sort the list
-        list.sort()
+        resultList.sort()
 
       category = ""
 
+      # TODO: replace with universal list from mod_menu
+
       # One option per row
       for row in (0,1,2): # TODO: dynamic adjustment (how to guess the screensize vs dpi ?)
         index = self.scroll + row
-        numItems = len(list)
+        if resultList:
+          numItems = len(resultList)
+        else:
+          numItems=0
         if(0 <= index < numItems):
 
-          (text1,text2,onClick) = self.describeItem(index, category, list)
+          (text1,text2,onClick) = self.describeItem(index, category, resultList)
 
           y = y4 + (row) * dy
           w = w1 - (x4-x1)
+          h = h1 / 3.0
 
           # Draw background and make clickable
           menus.drawButton(cr,
@@ -285,22 +306,21 @@ class search(ranaModule):
             "generic", # background for a 3x1 icon
             onClick)
 
-          border = 20
+          # result text
+          menus.drawText(cr, text1, x4+dx*0.10, y+dy*0.1, w-dx*0.20, h*0.5)
 
-          self.showText(cr, text1, x4+border, y+border, w-2*border)
-
-          # 2nd line: current value
-          self.showText(cr, text2, x4 + 0.15 * w, y + 0.6 * dy, w * 0.85 - border)
+          # 2nd line: distance to result
+          menus.drawText(cr, text2, x4+dx*0.15, y + 0.6 * dy, w * 0.3, 0.3 * dy, 0.05)
 
           # in corner: row number
-          self.showText(cr, "%d/%d" % (index+1, numItems), x4+0.85*w, y + 0.42 * dy, w * 0.15 - border, 20)
+          menus.drawText(cr, "%d/%d" % (index+1, numItems), x4+0.85*w, y + 0.7 * dy, w * 0.15, 0.2 * dy, 0.05)
 
     elif menuName == 'searchResultsItem':
       """draw the menu describing a single GLS result"""
-      list = self.updateDistance()
+      resultList = self.updateDistance()
 
       if self.get('GLSOrdering', 'default') == 'distance': # if ordering by distance is turned on, sort the list
-        list.sort()
+        resultList.sort()
 
       resultNumber = int(self.get('searchResultsItemNr', 0))
 
@@ -309,7 +329,7 @@ class search(ranaModule):
          which is created from the initial ordering
          without this,(with distance sort) we would get different results for a key, if we moved fast enoght :)
       """
-      result = self.getResult(resultNumber, list)
+      result = self.getResult(resultNumber, resultList)
       self.drawGLSResultMenu(cr, result)
 
     elif menuName == 'searchCustomQuery':
@@ -411,7 +431,11 @@ class search(ranaModule):
     lat = float(result['lat'])
     lon = float(result['lng'])
     units = self.m.get('units', None)
-    distanceString = units.km2CurrentUnitString(float(distance))
+    if units:
+      distanceString = units.km2CurrentUnitString(float(distance), dp=2)
+    else:
+      print "search: warning, the units module uis missing"
+      distanceString = "%1.2f km" % float(distance)
     # * draw "escape" button
     menus.drawButton(cr, x1, y1, dx, dy, "", "up", "search:reset|set:menu:searchResults")
     # * draw "show" button
@@ -495,7 +519,8 @@ class search(ranaModule):
 
       cr.set_font_size(20)
       extents = cr.text_extents(text) # get the text extents
-      (w,h) = (extents[2], extents[3])
+      (w,h) = (extents[2]*1.5, extents[3]*1.5)
+#      (w,h) = (extents[2], extents[3])
 
       border = 2
       cr.set_line_width(2)
@@ -509,11 +534,9 @@ class search(ranaModule):
       cr.fill()
       # draw the actual text
       cr.set_source_rgba(1, 1, 1, 0.95) # slightly trasparent white
-#      cr.move_to(x+10,y)
-      menus.drawText(cr, text, x+10, y, rw, rh, 2)
+      menus.drawText(cr, text, rx,ry-(-rh),rw,-rh, 0.05)
 #      cr.show_text(text) # show the trasparent result caption
       cr.stroke()
-      cr.fill()
 
     if highlightNr != -1: # is there some search result to highlight ?
       tupple = filter(lambda x: x[2] == int(highlightNr), self.list).pop()
@@ -559,13 +582,9 @@ class search(ranaModule):
       
       # draw the actual text
       cr.set_source_rgba(1, 1, 0, 0.95) # slightly trasparent white
-      cr.move_to(x+15,y+7)
-      cr.show_text(text) # show the trasparent result caption
+      menus.drawText(cr, text, rx,ry-(-rh),rw,-rh, 0.05)
       cr.stroke()
 
-  def firstTime(self):
-    self.menuWatchId = self.modrana.watch('menu', self._checkMenuEnteredCB)
-
   def _checkMenuEnteredCB(self, key, oldValue, newValue):
     if key == "menu" and newValue == "search":
       self.generateMenuStructure()
@@ -609,10 +628,31 @@ class search(ranaModule):
     if key == "customQuery":
       message = "ms:search:searchThis:%s" % result
       self.sendMessage(message)
-      self.set('menu', 'searchResults')
+#      self.set('menu', 'searchResults')
+    elif key == "address":
+      online = self.m.get('onlineServices')
+      textInput = result
+      if online:
+        # geocode the text input
+        results = online.geocode(textInput)
+        print("geocoding done - something found")
+        if results:
+          for r in results:
+            print(r)
+          place, (lat, lon) = results[0]
+          z = self.get('z', 15)
+          self.sendMessage('mapView:recentre %f %f %d|set:menu:None|ml:notification:m:%s;5' % (lat, lon, z, place))
+        else:
+          print("geocoding done - nothing found")
+          self.sendMessage('ml:notification:m:No results found for this address.;5')
+
+      else:
+        print("search: online services module missing")
 
   def handleSearchResult(self, key, results):
     if key == "localSearchResultGoogle":
       print "search: GLS result recieved"
       self.localSearchResults = results
-      self.set('menu', 'searchResults')       
\ No newline at end of file
+      self.set('menu', 'searchResults')
+    elif key == "address2LL":
+      print  "search address2LL received"
\ No newline at end of file
